@@ -132,46 +132,79 @@ QHash<int, QByteArray> PresetsModel::roleNames() const
     return roles;
 }
 
+QVector<QPair<QRegularExpression, QString>> KSysGuard::SensorFaceControllerPrivate::sensorIdReplacements;
 
-
-class SensorFaceController::Private
+SensorFaceControllerPrivate::SensorFaceControllerPrivate()
 {
-public:
-    Private();
-    SensorFace *createGui(const QString &qmlPath);
-    QQuickItem *createConfigUi(const QString &file, const QVariantMap &initialProperties);
+    if (SensorFaceControllerPrivate::sensorIdReplacements.isEmpty()) {
+        // A list of conversion rules to convert old sensor ids to new ones.
+        // When loading, each regular expression tries to match to the sensor
+        // id. If it matches, it will be be used to replace the sensor id with
+        // the second argument.
+        sensorIdReplacements = {
+            { QRegularExpression(QStringLiteral("network/interfaces/(.*)")), QStringLiteral("network/\\1")},
+            { QRegularExpression(QStringLiteral("network/all/receivedDataRate$")), QStringLiteral("network/all/download")},
+            { QRegularExpression(QStringLiteral("network/all/sentDataRate$")), QStringLiteral("network/all/upload")},
+            { QRegularExpression(QStringLiteral("network/all/totalReceivedData$")), QStringLiteral("network/all/totalDownload")},
+            { QRegularExpression(QStringLiteral("network/all/totalSentData$")), QStringLiteral("network/all/totalUpload")},
+            { QRegularExpression(QStringLiteral("(.*)/receiver/data$")), QStringLiteral("\\1/download")},
+            { QRegularExpression(QStringLiteral("(.*)/transmitter/data$")), QStringLiteral("\\1/upload")},
+            { QRegularExpression(QStringLiteral("(.*)/receiver/dataTotal$")), QStringLiteral("\\1/totalDownload")},
+            { QRegularExpression(QStringLiteral("(.*)/transmitter/dataTotal$")), QStringLiteral("\\1/totalUpload")},
+        };
+    }
+}
 
-    SensorFaceController *q;
-    QString title;
-    QQmlEngine *engine;
+QJsonArray SensorFaceControllerPrivate::readSensors(const KConfigGroup &read, const QString &entryName)
+{
+    auto original = QJsonDocument::fromJson(read.readEntry(entryName, QString()).toUtf8()).array();
+    QJsonArray newSensors;
+    for (auto entry : original) {
+        QString sensorId = entry.toString();
+        for (auto replacement : qAsConst(sensorIdReplacements)) {
+            auto match = replacement.first.match(sensorId);
+            if (match.hasMatch()) {
+                sensorId.replace(replacement.first, replacement.second);
+            }
+        }
+        newSensors.append(sensorId);
+    }
 
-    KConfigGroup faceProperties;
-    KDeclarative::ConfigPropertyMap *faceConfiguration = nullptr;
-    KConfigLoader *faceConfigLoader = nullptr;
+    return newSensors;
+}
 
-    bool configNeedsSave = false;
-    KPackage::Package facePackage;
-    QString faceId;
-    KLocalizedContext *contextObj = nullptr;
-    KConfigGroup configGroup;
-    KConfigGroup appearanceGroup;
-    KConfigGroup sensorsGroup;
-    KConfigGroup colorsGroup;
-    QPointer <SensorFace> fullRepresentation;
-    QPointer <SensorFace> compactRepresentation;
-    QPointer <QQuickItem> faceConfigUi;
-    QPointer <QQuickItem> appearanceConfigUi;
-    QPointer <QQuickItem> sensorsConfigUi;
+QJsonArray SensorFaceControllerPrivate::readAndUpdateSensors(KConfigGroup& config, const QString& entryName)
+{
+    auto original = QJsonDocument::fromJson(config.readEntry(entryName, QString()).toUtf8()).array();
 
-    QTimer *syncTimer;
-    FacesModel *availableFacesModel = nullptr;
-    PresetsModel *availablePresetsModel = nullptr;
+    const KConfigGroup &group = config;
+    auto newSensors = readSensors(group, entryName);
+
+    if (newSensors != original) {
+        config.writeEntry(entryName, QJsonDocument(newSensors).toJson(QJsonDocument::Compact));
+    }
+
+    return newSensors;
+}
+
+QJsonArray SensorFaceControllerPrivate::resolveSensors(const QJsonArray &partialEntries)
+{
+    QJsonArray sensors;
+
+    for (const auto &id : partialEntries) {
+        KSysGuard::SensorQuery query{id.toString()};
+        query.execute();
+        query.waitForFinished();
+        auto ids = query.sensorIds();
+        std::stable_sort(ids.begin(), ids.end());
+        for (const auto &fitleredId : qAsConst(ids)) {
+            sensors.append(QJsonValue(fitleredId));
+        }
+    }
+    return sensors;
 };
 
-SensorFaceController::Private::Private()
-{}
-
-SensorFace *SensorFaceController::Private::createGui(const QString &qmlPath)
+SensorFace *SensorFaceControllerPrivate::createGui(const QString &qmlPath)
 {
     QQmlComponent *component = new QQmlComponent(engine, qmlPath, nullptr);
     // TODO: eventually support async  components? (only useful for qml files from http, we probably don't want that)
@@ -204,7 +237,7 @@ SensorFace *SensorFaceController::Private::createGui(const QString &qmlPath)
     return gui;
 }
 
-QQuickItem *SensorFaceController::Private::createConfigUi(const QString &file, const QVariantMap &initialProperties)
+QQuickItem *SensorFaceControllerPrivate::createConfigUi(const QString &file, const QVariantMap &initialProperties)
 {
     QQmlComponent *component = new QQmlComponent(engine, file, nullptr);
     // TODO: eventually support async  components? (only useful for qml files from http, we probably don't want that)
@@ -231,12 +264,9 @@ QQuickItem *SensorFaceController::Private::createConfigUi(const QString &file, c
 }
 
 
-
-
-
 SensorFaceController::SensorFaceController(KConfigGroup &config, QQmlEngine *engine)
     : QObject(engine),
-      d(std::make_unique<Private>())
+      d(std::make_unique<SensorFaceControllerPrivate>())
 {
     d->q = this;
     d->configGroup = config;
@@ -248,10 +278,19 @@ SensorFaceController::SensorFaceController(KConfigGroup &config, QQmlEngine *eng
     d->syncTimer->setSingleShot(true);
     d->syncTimer->setInterval(5000);
     connect(d->syncTimer, &QTimer::timeout, this, [this]() {
+        if (!d->shouldSync) {
+            return;
+        }
         d->appearanceGroup.sync();
         d->sensorsGroup.sync();
     });
 
+    d->contextObj = new KLocalizedContext(this);
+
+    d->totalSensors = d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("totalSensors")));
+    d->lowPrioritySensorIds = d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("lowPrioritySensorIds")));
+    d->highPrioritySensorIds = d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("highPrioritySensorIds")));
+
     setFaceId(d->appearanceGroup.readEntry("chartFace", QStringLiteral("org.kde.ksysguard.piechart")));
 }
 
@@ -278,15 +317,17 @@ void SensorFaceController::setTitle(const QString &title)
 
 QJsonArray SensorFaceController::totalSensors() const
 {
-    QJsonDocument doc = QJsonDocument::fromJson(d->sensorsGroup.readEntry("totalSensors", QString()).toUtf8());
-    return doc.array();
+    return d->totalSensors;
 }
 
 void  SensorFaceController::setTotalSensors(const QJsonArray &totalSensors)
 {
-    if (totalSensors == SensorFaceController::totalSensors()) {
+    QJsonArray resolvedSensors = d->resolveSensors(totalSensors);
+
+    if (resolvedSensors == d->totalSensors) {
         return;
     }
+    d->totalSensors = resolvedSensors;
 
     d->sensorsGroup.writeEntry("totalSensors", QJsonDocument(totalSensors).toJson(QJsonDocument::Compact));
     d->syncTimer->start();
@@ -295,16 +336,18 @@ void  SensorFaceController::setTotalSensors(const QJsonArray &totalSensors)
 
 QJsonArray SensorFaceController::highPrioritySensorIds() const
 {
-    QJsonDocument doc = QJsonDocument::fromJson(d->sensorsGroup.readEntry("highPrioritySensorIds", QString()).toUtf8());
-    return doc.array();
+    return d->highPrioritySensorIds;
 }
 
 
 void SensorFaceController::setHighPrioritySensorIds(const QJsonArray &highPrioritySensorIds)
 {
-    if (highPrioritySensorIds == SensorFaceController::highPrioritySensorIds()) {
+    QJsonArray resolvedSensors = d->resolveSensors(highPrioritySensorIds);
+
+    if (resolvedSensors == d->highPrioritySensorIds) {
         return;
     }
+    d->highPrioritySensorIds = resolvedSensors;
 
     d->sensorsGroup.writeEntry("highPrioritySensorIds", QJsonDocument(highPrioritySensorIds).toJson(QJsonDocument::Compact));
     d->syncTimer->start();
@@ -326,6 +369,10 @@ QVariantMap SensorFaceController::sensorColors() const
 
 void SensorFaceController::setSensorColors(const QVariantMap &colors)
 {
+    if (colors == this->sensorColors()) {
+        return;
+    }
+
     d->colorsGroup.deleteGroup();
     d->colorsGroup = KConfigGroup(&d->configGroup, "SensorColors");
 
@@ -340,15 +387,16 @@ void SensorFaceController::setSensorColors(const QVariantMap &colors)
 
 QJsonArray SensorFaceController::lowPrioritySensorIds() const
 {
-    QJsonDocument doc = QJsonDocument::fromJson(d->sensorsGroup.readEntry("lowPrioritySensorIds", QString()).toUtf8());
-    return doc.array();
+    return d->lowPrioritySensorIds;
 }
 
 void SensorFaceController::setLowPrioritySensorIds(const QJsonArray &lowPrioritySensorIds)
 {
-    if (lowPrioritySensorIds == SensorFaceController::lowPrioritySensorIds()) {
+    QJsonArray resolvedSensors = d->resolveSensors(lowPrioritySensorIds);
+    if (resolvedSensors == d->lowPrioritySensorIds) {
         return;
     }
+    d->lowPrioritySensorIds = lowPrioritySensorIds;
 
     d->sensorsGroup.writeEntry("lowPrioritySensorIds", QJsonDocument(lowPrioritySensorIds).toJson(QJsonDocument::Compact));
     d->syncTimer->start();
@@ -381,6 +429,11 @@ bool SensorFaceController::supportsLowPrioritySensors() const
     return d->faceProperties.readEntry("SupportsLowPrioritySensors", false);
 }
 
+int SensorFaceController::maxTotalSensors() const
+{
+    return d->faceProperties.readEntry("MaxTotalSensors", 1);
+}
+
 void SensorFaceController::setFaceId(const QString &face)
 {
     if (d->faceId == face) {
@@ -404,7 +457,6 @@ void SensorFaceController::setFaceId(const QString &face)
 
     d->facePackage = KPackage::PackageLoader::self()->loadPackage(QStringLiteral("KSysguard/SensorFace"), face);
 
-    d->contextObj->deleteLater();
     if (d->faceConfiguration) {
         d->faceConfiguration->deleteLater();
         d->faceConfiguration = nullptr;
@@ -419,7 +471,6 @@ void SensorFaceController::setFaceId(const QString &face)
         return;
     }
 
-    d->contextObj = new KLocalizedContext(d->engine);
     d->contextObj->setTranslationDomain(QLatin1String("ksysguard_face_") + face);
 
     d->faceProperties = KConfigGroup(KSharedConfig::openConfig(d->facePackage.filePath("FaceProperties")), QStringLiteral("Config"));
@@ -432,6 +483,7 @@ void SensorFaceController::setFaceId(const QString &face)
 
         d->faceConfigLoader = new KConfigLoader(cg, &file, this);
         d->faceConfiguration = new KDeclarative::ConfigPropertyMap(d->faceConfigLoader, this);
+        d->faceConfiguration->setAutosave(d->shouldSync);
     }
 
     d->appearanceGroup.writeEntry("chartFace", face);
@@ -471,7 +523,6 @@ QQuickItem *SensorFaceController::fullRepresentation()
     }
 
     d->fullRepresentation = d->createGui(d->facePackage.filePath("ui", QStringLiteral("FullRepresentation.qml")));
-
     return d->fullRepresentation;
 }
 
@@ -579,8 +630,8 @@ void SensorFaceController::loadPreset(const QString &preset)
     KDesktopFile df(presetPackage.path() + QStringLiteral("metadata.desktop"));
 
     auto c = KSharedConfig::openConfig(presetPackage.filePath("config", QStringLiteral("faceproperties")));
-    KConfigGroup presetGroup(c, QStringLiteral("Config"));
-    KConfigGroup colorsGroup(c, QStringLiteral("SensorColors"));
+    const KConfigGroup presetGroup(c, QStringLiteral("Config"));
+    const KConfigGroup colorsGroup(c, QStringLiteral("SensorColors"));
 
     // Load the title
     setTitle(df.readName());
@@ -591,30 +642,9 @@ void SensorFaceController::loadPreset(const QString &preset)
         d->availablePresetsModel->removeRow(0);
     }
 
-    auto loadSensors = [this](const QJsonArray &partialEntries) {
-        QJsonArray sensors;
-
-        for (const auto &id : partialEntries) {
-            KSysGuard::SensorQuery query{id.toString()};
-            query.execute();
-            query.waitForFinished();
-            auto ids = query.sensorIds();
-            std::stable_sort(ids.begin(), ids.end());
-            for (const auto &fitleredId : qAsConst(ids)) {
-                sensors.append(QJsonValue(fitleredId));
-            }
-        }
-        return sensors;
-    };
-
-    QJsonDocument doc = QJsonDocument::fromJson(presetGroup.readEntry("totalSensors", QString()).toUtf8());
-    setTotalSensors(loadSensors(doc.array()));
-
-    doc = QJsonDocument::fromJson(presetGroup.readEntry("highPrioritySensorIds", QString()).toUtf8());
-    setHighPrioritySensorIds(loadSensors(doc.array()));
-
-    doc = QJsonDocument::fromJson(presetGroup.readEntry("lowPrioritySensorIds", QString()).toUtf8());
-    setLowPrioritySensorIds(loadSensors(doc.array()));
+    setTotalSensors(d->readSensors(presetGroup, QStringLiteral("totalSensors")));
+    setHighPrioritySensorIds(d->readSensors(presetGroup, QStringLiteral("highPrioritySensorIds")));
+    setLowPrioritySensorIds(d->readSensors(presetGroup, QStringLiteral("lowPrioritySensorIds")));
 
     setFaceId(presetGroup.readEntry(QStringLiteral("chartFace"), QStringLiteral("org.kde.ksysguard.piechart")));
 
@@ -661,7 +691,7 @@ void SensorFaceController::savePreset()
         return;
     }
 
-    KConfig c(dir.path() % QLatin1Literal("/metadata.desktop"));
+    KConfig c(dir.path() % QStringLiteral("/metadata.desktop"));
 
     KConfigGroup cg(&c, "Desktop Entry");
     cg.writeEntry("Name", title());
@@ -680,7 +710,7 @@ void SensorFaceController::savePreset()
 
     QDir subDir(dir.path());
     subDir.mkdir(QStringLiteral("contents"));
-    KConfig faceConfig(subDir.path() % QLatin1Literal("/contents/faceproperties"));
+    KConfig faceConfig(subDir.path() % QStringLiteral("/contents/faceproperties"));
 
     KConfigGroup configGroup(&faceConfig, "Config");
     configGroup.writeEntry(QStringLiteral("totalSensors"), QJsonDocument(totalSensors()).toJson(QJsonDocument::Compact));
@@ -724,4 +754,21 @@ void SensorFaceController::uninstallPreset(const QString &pluginId)
     });
 }
 
+bool SensorFaceController::shouldSync() const
+{
+    return d->shouldSync;
+}
+
+void SensorFaceController::setShouldSync(bool sync)
+{
+    d->shouldSync = sync;
+    if (d->faceConfiguration) {
+        d->faceConfiguration->setAutosave(sync);
+    }
+    if (!d->shouldSync && d->syncTimer->isActive()) {
+        d->syncTimer->stop();
+    }
+}
+
+
 #include "moc_SensorFaceController.cpp"
